/* Thunderbird Plugin: Create Jira Issue
 *
 * This Plugin is able to create Jira-Issues out of an email.
 * Furthermore the content of an email can be added to an issue
 * as a comment.
 *
 * Requirements:
 * - Jira 4.0 and above with enabled SOAP-API
 * - Thunderbird 3.0+
 *
 * Author: catWorkX GmbH, Hamburg
 * 		   Sebastian Gerdes <sebastian.gerdes_AT_catworkx.de>
 *
 */

// Called once when the dialog displays
var projects;
var token;
var issuetypesReturn;
var soapurl;
var timerid;

function onLoad(){
    // Use the arguments passed to us by the caller
    document.getElementById("issueSubject").value = window.arguments[0].issueSubject;
    createjiraissue.setCheckboxAdditionalHeader();
	createjiraissue.setCheckboxAssignToMe();
	projects = window.arguments[1];
	var selectProjectFound = false;

	if (projects.length == 0) {
		var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
        promptService.alert(window, "Error", "You do not have the permission to browse any projects.");
		window.close();
	}

    // add menu item to project menu
    var projectMenu = document.getElementById("jiraprojectsmenupopup");
    var menuList = document.getElementById("jiraprojectsmenulist");
    var tempItem;
    var selectedproject = window.arguments[2];

    for (var i = 0; i < projects.length; i++) {
        tempItem = document.createElement("menuitem");
        // set project name and key as label
        tempItem.setAttribute("label", projects[i][0] + " (" + projects[i][1] + ")");
        // add project ID as value attribute
        tempItem.setAttribute("value", i);
        tempItem.setAttribute("projKey", projects[i][1]);
		tempItem.setAttribute("projId", projects[i][2]);
        projectMenu.appendChild(tempItem);



        // set the default based on mapping
        if (selectedproject == projects[i][1]) {
             menuList.selectedIndex = i;
			 selectProjectFound = true;
        }
    }
	if (!selectProjectFound)
		menuList.selectedIndex = 0;
}

function modifyDescriptionOrSummary() {
	clearTimeout(timerid);
	timerid = setTimeout(function() { setAcceptButtonStatus(); }, 250);
}


function setAcceptButtonStatus() {
	var projMenulist = document.getElementById("jiraprojectsmenulist").selectedItem;
	var itMenulist = document.getElementById("jiraprojectsmenulist").selectedItem;
	var issueSubject = document.getElementById("issueSubject").value;
    var issueDescription = document.getElementById("issueDescription").value;
	if (projMenulist != null && itMenulist != null && issueSubject != "" && issueDescription != "") {
		document.documentElement.getButton('accept').disabled = false;
	}
	else {
		document.documentElement.getButton('accept').disabled = true;
	}
}

function projectSelected(){
	token = window.arguments[0].token;
    soapurl = window.arguments[0].soapurl;
	issuetypesReturn = [];

    var projId = document.getElementById("jiraprojectsmenulist").selectedItem.getAttribute("projId");

    // get current selected project
    //var curProjId = document.getElementById("jiraprojectsmenulist").selectedItem.getAttribute("value");

	// SOAP call
    if (projId != null && projId != "") {
		jiraLoadIssuetypes(projId);
		var issuetypes = issuetypesReturn.sort();

		// remove all current items
		var issuetypeMenu = document.getElementById("jiraissuetypemenupopup");
		while (issuetypeMenu.hasChildNodes()) {
			issuetypeMenu.removeChild(issuetypeMenu.firstChild);
		}

		if (issuetypes != null) {
			var tempItem;
			for (var i = 0; i < issuetypes.length; i++) {
				tempItem = document.createElement("menuitem");
				// set issue type name as label
				tempItem.setAttribute("label", issuetypes[i][0]);
				// add issue type ID as value attribute
				tempItem.setAttribute("value", issuetypes[i][1]);
				issuetypeMenu.appendChild(tempItem);
			}
			// select first item
			document.getElementById("jiraissuetypemenulist").selectedIndex = 0;
			document.getElementById("jiraissuetypemenulist").disabled = false;
		}
	}

	setAcceptButtonStatus();
}

function jiraLoadIssuetypes(projId){
		var pl = new SOAPClientParameters();
		pl.add("token", token);
		pl.add("project", projId);
		SOAPClient.invoke(soapurl, "getIssueTypesForProject", pl, false, parseIssuetypeResult);
}

function parseIssuetypeResult(result, xml){
    //createjiraissue.printXml(xml);

    // get the IDs of all of the issuetypes objects
    var issuetypesXmlIds = new Array();
    var hrefs = xml.getElementsByTagName("getIssueTypesForProjectReturn");
    for (var i = 0; i < hrefs.length; i++) {
        var attrs = hrefs[i].attributes;
        for (var j = 0; j < attrs.length; j++) {
            if (attrs[j].name == "href") {
                issuetypesXmlIds[i] = (new String(attrs[j].value)).substr(1);

            }
        }
    }

    var issuetypes = xml.getElementsByTagName("multiRef");
    var issuetypeSize = 0;

    for (var k = 0; k < issuetypes.length; k++) {
        var issuetype = issuetypes[k];
        var itAttrs = issuetype.attributes;
        var multirefId = "";
        for (var m = 0; m < itAttrs.length; m++) {
            if (itAttrs[m].name == "id") {
                multirefId = itAttrs[m].value;
            }
        }
        for (var n = 0; n < issuetypesXmlIds.length; n++) {
            if (multirefId == issuetypesXmlIds[n]) {
                // got a real issuetype
                var itId = issuetype.getElementsByTagName("id").item(0).firstChild.nodeValue;
                var itName = issuetype.getElementsByTagName("name").item(0).firstChild.nodeValue;
                //createjiraissue.LOG("Got issuetype with id: " + itId + " and name: " + itName);
                issuetypesReturn[issuetypeSize++] = new Array(itName, itId );
            }
        }
    }
}

// called if the user clicks "create issue"
function doSave(){
    // Return the changed arguments
    window.arguments[0].out = {
    	issueSubject: document.getElementById("issueSubject").value,
        issueDescription: document.getElementById("issueDescription").value,
        projId: document.getElementById("jiraprojectsmenulist").selectedItem.getAttribute("projKey"),
        itId: document.getElementById("jiraissuetypemenulist").selectedItem.getAttribute("value"),
		assignToMe: document.getElementById("checkboxAssignToMe").checked
    };
    return true;
}

// called if the user clicks cancel
function doCancel(){
    return true;
}

