/* Thunderbird Plugin: Create Jira Issue
 *
 * This Plugin is able to create Jira-Issues out of an email.
 * Furthermore the content of an email can be added to an issue
 * as a comment.
 *
 * Requirements:
 * - Jira 4.0 and above with enabled SOAP-API
 * - Thunderbird 3.0+
 *
 * Author: catWorkX GmbH, Hamburg
 * 		   Sebastian Gerdes <sebastian.gerdes_AT_catworkx.de>
 *
 */
var token;
var soapurl;
var projects;
var issues;
var timerid;

// Called once when the dialog displays
function onLoad(){
    // Use the arguments passed to us by the caller
	createjiraissue.setCheckboxAdditionalHeader();
    projects = window.arguments[1];
	var selectProjectFound = false;

	if (projects.length == 0) {
		var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
        promptService.alert(window, "Error", "You do not have the permission to browse any projects.");
		window.close();
	}

    // global variable for issues
    issues = new Array();
    // add menuitem to project menu
    var projectMenu = document.getElementById("jiraprojectsmenupopup");
	var menuList = document.getElementById("jiraprojectsmenulist");
    var tempItem;
	var selectedproject = window.arguments[2];

    for (var i = 0; i < projects.length; i++) {
        tempItem = document.createElement("menuitem");
        // set project name and key as label
        tempItem.setAttribute("label", projects[i][0] + " (" + projects[i][1] + ")");
        // add project ID as value attribute;
        tempItem.setAttribute("value", i);
        tempItem.setAttribute("projId", projects[i][1]);
        tempItem.setAttribute("projName", projects[i][0]);
        projectMenu.appendChild(tempItem);

		//set the default based on mapping
        if (selectedproject == projects[i][1]) {
             menuList.selectedIndex = i + 1;
			 selectProjectFound = true;
        }
    }

	if (!selectProjectFound)
		menuList.selectedIndex = 0;

}


function setAcceptButtonStatus() {
	//var projValue = document.getElementById("jiraprojectsmenulist").selectedItem.getAttribute("value");
	var description = document.getElementById("issueDescription").value;
	if (document.getElementById("jiraissuemenulist").selectedItem != null && description != "") {
		document.documentElement.getButton('accept').disabled = false;
	}
	else {
		document.documentElement.getButton('accept').disabled = true;
	}
}

function enterSearchpattern() {
	clearTimeout(timerid);
	timerid = setTimeout(function() { projectSelected(); }, 250);

}

function modifyDescription() {
	clearTimeout(timerid);
	timerid = setTimeout(function() { setAcceptButtonStatus(); }, 250);
}

function projectSelected(){

    token = window.arguments[0].token;
    soapurl = window.arguments[0].soapurl;
    var projName = document.getElementById("jiraprojectsmenulist").selectedItem.getAttribute("projName");
	//var pattern = document.getElementById("issueSearchPattern").value;
    var issueMenu = document.getElementById("jiraissuemenulist");

    // remove all current items
    removeAllItemsFromIssueMenuList();

    // SOAP call
    if (projName != null && projName != 'undefined') {
        jiraLoadIssues(projName);

        if (issues != 'undefined' && issues != null && issues.length != 0) {
            var tempItem;
            for (var i = 0; i < issues.length; i++) {
                tempItem = document.createElement("listitem");
                // set issue summary as label
                tempItem.setAttribute("label", issues[i][0] + " (" + issues[i][1] + ")");
                // add issue key as key attribute
                tempItem.setAttribute("key", issues[i][1]);
                // add issue ID as value attribute
                tempItem.setAttribute("value", issues[i][2]);
                issueMenu.appendChild(tempItem);
            }
        }
        else {
            // create empty item to clear the issue list
            var tempItem;
            tempItem = document.createElement("listitem");
            issueMenu.appendChild(tempItem);
        }
    }
}

function jiraLoadIssues(projName){

    var pattern = document.getElementById("issueSearchPattern").value;
    var patternKey = document.getElementById("issueSearchKey").value;
	var jqlSearch = "";
    //createjiraissue.LOG("regex match: " + patternKey.search(/[a-zA-Z]+-[0-9]+/));
    if (patternKey != null && patternKey != "" && patternKey != 'undefined' && patternKey.search(/[a-zA-Z]+-[0-9]+/) != -1) {
        jqlSearch = "issuekey = \"" + patternKey + "\"";
        doJqlSearch(jqlSearch);
    } else if (pattern != null && pattern != "" && pattern != 'undefined') {
		pattern = pattern.trim();
		if (document.getElementById("jiraprojectsmenulist").selectedItem.getAttribute("value") != -1)
        	jqlSearch += "project = \"" + projName + "\" AND ";
		jqlSearch += "(summary ~ \"" + pattern + "*\"";
        if (document.getElementById("checkboxScopeDescription").checked == true)
            jqlSearch += " OR description ~ \"" + pattern + "*\"";
        if (document.getElementById("checkboxScopeComment").checked == true)
            jqlSearch += " OR comment ~ \"" + pattern + "*\"";
        jqlSearch += ") AND (status = \"Open\"";
        if (document.getElementById("checkboxStatusProgress").checked == true)
            jqlSearch += " OR status = \"In Progress\"";
        if (document.getElementById("checkboxStatusReopened").checked == true)
            jqlSearch += " OR status = \"Reopened\"";
        if (document.getElementById("checkboxStatusResolved").checked == true)
            jqlSearch += " OR status = \"Resolved\"";
        if (document.getElementById("checkboxStatusClosed").checked == true)
            jqlSearch += " OR status = \"Closed\"";
        jqlSearch += ") ORDER BY issuekey DESC";
        doJqlSearch(jqlSearch);
    }

}

function doJqlSearch(jqlSearch){
    try {
        //createjiraissue.LOG(jqlSearch);
        var pl = new SOAPClientParameters();
        pl.add("token", token);
        pl.add("jqlSearch", jqlSearch);
        pl.add("maxNumResults", 1000);
        SOAPClient.invoke(soapurl, "getIssuesFromJqlSearch", pl, false, parsejiraLoadIssuesResult);
    } catch (e) {
        issues = [];
    }

}

function removeAllItemsFromIssueMenuList() {
    var issueMenu = document.getElementById("jiraissuemenulist");
    while (issueMenu.hasChildNodes()) {
    	issueMenu.removeChild(issueMenu.firstChild);
    }
}

function parsejiraLoadIssuesResult(result, xml){
    //createjiraissue.printXml(xml);

    // get the IDs of all of the project objects
    var issueXmlIds = new Array();
    var hrefs = xml.getElementsByTagName("getIssuesFromJqlSearchReturn");
    for (var i = 0; i < hrefs.length; i++) {
        var attrs = hrefs[i].attributes;
        for (var j = 0; j < attrs.length; j++) {
            if (attrs[j].name == "href") {
                issueXmlIds[i] = (new String(attrs[j].value)).substr(1);
            }
        }
    }

    var localissues = xml.getElementsByTagName("multiRef");
    var issueKeySize = 0;
    // clear issues
	issues = [];

    for (var k = 0; k < localissues.length; k++) {
        var issue = localissues[k];
        var issueAttrs = issue.attributes;
        var multirefId = "";
        for (var m = 0; m < issueAttrs.length; m++) {
            if (issueAttrs[m].name == "id") {
                multirefId = issueAttrs[m].value;
            }
        }
        for (var n = 0; n < issueXmlIds.length; n++) {
            if (multirefId == issueXmlIds[n]) {
                // got a real project
                var issueId = issue.getElementsByTagName("id").item(0).firstChild.nodeValue;
                var issueKey = issue.getElementsByTagName("key").item(0).firstChild.nodeValue;
                var issueSummary = issue.getElementsByTagName("summary").item(0).firstChild.nodeValue;
                //createjiraissue.LOG("DIALOGCOMMENT: Got issue with key: " + issueKey + ", id: " + issueId + " and summary: " + issueSummary);
                issues[issueKeySize++] = new Array(issueSummary, issueKey, issueId);
            }
        }
    }
}

// called if the user clicks "add as comment"
function doSave(){
    // Return the changed arguments
	var issueKey = document.getElementById("jiraissuemenulist").selectedItem.getAttribute("key");
	//get preferences
    prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    prefs = prefs.getBranch("extensions.createjiraissue.");
    lastissues = prefs.getCharPref("lastissues");
    var lastissuesArray = lastissues.split(",");
    if ( lastissuesArray.indexOf(issueKey) == -1 ) {
    	lastissuesArray.unshift(issueKey);
    	if ( lastissuesArray.length > 10 ) {
    		lastissuesArray = lastissuesArray.slice(0,10);
    	}
    }
    lastissues = lastissuesArray.join(",");
    if ( lastissues.charAt(0) == "," ) {
    	lastissues = lastissues.substring(1); // leave out 2nd param to get all chars up to the end
    }
	prefs.setCharPref("lastissues",lastissues);
	window.arguments[0].out = {
		issueDescription: document.getElementById("issueDescription").value,
		issueKey: issueKey
	};
		return true;
}

// called if the user clicks cancel
function doCancel(){
    return true;
}

