/* Thunderbird Plugin: Create Jira Issue
 *
 * This Plugin is able to create Jira-Issues out of an email.
 * Furthermore the content of an email can be added to an issue
 * as a comment.
 *
 * Requirements:
 * - Jira 4.0 and above with enabled SOAP-API
 * - Thunderbird 3.0+
 *
 * Author: catWorkX GmbH, Hamburg
 * 		   Sebastian Gerdes <sebastian.gerdes_AT_catworkx.de>
 *
 */
var error = false;

var createjiraissue = {

    init: function(){
		createjiraissue.loadInitData();
		if (error)
			return;

		// open dialog windows and pass projects, issue types and mail content
        var params = {
            issueSubject: createjiraissue.mailsubject,
            issueDescription: createjiraissue.mailbody,
			issueDescriptionPrefix: createjiraissue.mailbodyprefix,
			token: createjiraissue.token,
            soapurl: createjiraissue.soapurl,
            out: null
        };
        window.openDialog("chrome://createjiraissue/content/dialog.xul", "createIssueDialog", "chrome, dialog, modal, resizable=yes", params, createjiraissue.projects, createjiraissue.selectedproject).focus();

        if (params.out) {
            //createjiraissue.promptService.alert(window, "Return", "subject" + params.out.subject + "projId" + params.out.projId + "itId" + params.out.itId);
            createjiraissue.jiraCreateIssue(params.out.issueSubject, params.out.issueDescription, params.out.projId, params.out.itId, params.out.assignToMe);
            // logout
            createjiraissue.jiraLogout();
        }
    },

    initComment: function(){
       	createjiraissue.loadInitData();
		if (error)
			return;

        var params = {
            issueDescription: createjiraissue.mailbody,
			issueDescriptionPrefix: createjiraissue.mailbodyprefix,
            token: createjiraissue.token,
            soapurl: createjiraissue.soapurl,
            out: null
        };

        window.openDialog("chrome://createjiraissue/content/dialogComment.xul", "commentDialog", "chrome, dialog, modal, resizable=yes", params, createjiraissue.projects, createjiraissue.selectedproject).focus();

        if (params.out) {
            //createjiraissue.promptService.alert(window, "Return", "ok click");
            createjiraissue.jiraAddComment(params.out.issueDescription, params.out.issueKey);
            // logout
            createjiraissue.jiraLogout();
        }
    },

	loadInitData: function(){
		createjiraissue.strings = document.getElementById("createjiraissue-strings");
        createjiraissue.promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
        createjiraissue.alertService = Components.classes["@mozilla.org/alerts-service;1"].getService(Components.interfaces.nsIAlertsService);

		// get preferences
		createjiraissue.prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    	createjiraissue.prefs = createjiraissue.prefs.getBranch("extensions.createjiraissue.");
        createjiraissue.jiraurl = createjiraissue.prefs.getCharPref("jiraurl");
		createjiraissue.projectmappingstring = createjiraissue.prefs.getCharPref("projectmapping");

		// get username and password from password manager
		var loginManager = Components.classes["@mozilla.org/login-manager;1"].getService(Components.interfaces.nsILoginManager);
		//var nsLoginInfo = new Components.Constructor("@mozilla.org/login-manager/loginInfo;1", Components.interfaces.nsILoginInfo, "init");
		var logins = loginManager.findLogins({}, "chrome://createjiraissue", null, "Jira Login");
		if (logins.length > 0) {
			createjiraissue.username = logins[0].username;
			createjiraissue.password = logins[0].password;
			error = false;
		} else {
            window.openDialog("chrome://createjiraissue/content/options.xul", "createjiraissuePreferences", "chrome, dialog, modal, resizable=yes").focus();
			error = true;
			return;
        }

        createjiraissue.soapurl = createjiraissue.jiraurl + "/rpc/soap/jirasoapservice-v2";

		var mappingArray = new Array();
        var lines = createjiraissue.projectmappingstring.split('\n');
        for (var i = 0; i < lines.length; ++i) {
            var l = lines[i].split('|');
            mappingArray[i] = l;
        }
        createjiraissue.projectmapping = mappingArray;
		createjiraissue.getMailContent();
        createjiraissue.token = null;
        createjiraissue.projCounter = 0;
        createjiraissue.issuetypes = new Array();
        createjiraissue.jiraLogin();

		if (createjiraissue.token == null) {
			error = true;
			return false;
		}

        // load Jira projects
        createjiraissue.jiraLoadProjects();

        // sort array
        createjiraissue.projects.sort();

        // get preselected project
		createjiraissue.selectedproject = "";
        for (var x = 0; x < mappingArray.length; x++) {
            if (createjiraissue.mailfrom.search(mappingArray[x][0]) != -1){
                 createjiraissue.selectedproject = mappingArray[x][1];
            }
        }
	},

    getMailContent: function(){

        var MessageURI = gFolderDisplay.selectedMessageUris[0];
        var msg = messenger.messageServiceFromURI(MessageURI);
        var header = msg.messageURIToMsgHdr(MessageURI);

        var subject = header.mime2DecodedSubject;
        var from = header.mime2DecodedAuthor;
        var to = header.mime2DecodedRecipients;
		var cclist = header.ccList.replace(",",",\n\t\t");
		var bcclist = header.bccList.replace(",",",\n\t\t");

        var msgdate = new Date( header.date/1000);

        createjiraissue.mailsubject = header.mime2DecodedSubject;
        createjiraissue.mailfrom = header.author;

        createjiraissue.mailbodyprefix = 'Date:\t\t' + msgdate + '\n' + 'Subject:\t' + subject + '\n' + 'From:\t' + from + '\n' + 'To:\t\t' + to;
		if (cclist != "")
			createjiraissue.mailbodyprefix = createjiraissue.mailbodyprefix + '\n' + 'Cc:\t\t' + cclist;
		if (bcclist != "")
			createjiraissue.mailbodyprefix = createjiraissue.mailbodyprefix + '\n' + 'Bcc:\t\t' + bcclist;
		createjiraissue.mailbodyprefix = createjiraissue.mailbodyprefix +
		'\n--------------------------------------------------------------------------------------------\n\n';
        var doc = document.getElementById("messagepane").contentDocument;
        var mailbodyhtml = doc.body.innerHTML;
        var selection = doc.getSelection();
        if(selection != null && selection.toString().length > 0) {
            mailbodyhtml = selection.toString();
        }
        //createjiraissue.LOG(createjiraissue.mailbodyhtml);

        // remove html tags
        createjiraissue.mailbody = mailbodyhtml.replace(/<([^>]+)>/g, '');
        // trim
        createjiraissue.mailbody = createjiraissue.mailbody.replace(/^\s*|\s*$/g, "");
        // remove extra newlines
        // createjiraissue.mailbody = createjiraissue.mailbody.replace(/[\n\r]+/g, "\n");
        // render some html entities
        createjiraissue.mailbody = createjiraissue.mailbody.replace(/\&gt;/g, ">").replace(/\&lt;/g, "<").replace(/\&nbsp;/g, " ");
        //createjiraissue.LOG(createjiraissue.mailbody);
    },

    jiraLogin: function(){
		try {
			var pl = new SOAPClientParameters();
        	pl.add("username", createjiraissue.username);
        	pl.add("password", createjiraissue.password);
        	SOAPClient.invoke(createjiraissue.soapurl, "login", pl, false, createjiraissue.parseLoginResult);
		} catch (e) {
			createjiraissue.promptService.alert(window, "Error", "Login to Jira failed.");
		}
    },

    parseLoginResult: function(r){
        //createjiraissue.LOG("token " + r);
        if (r == null) {
            createjiraissue.LOG("Failed to log in to " + createjiraissue.jiraurl);
            return;
        }
        createjiraissue.token = r;
    },

    jiraLogout: function(){
        var pl = new SOAPClientParameters();
        pl.add("token", createjiraissue.token);
        SOAPClient.invoke(createjiraissue.soapurl, "logout", pl, false);
    },

    jiraLoadProjects: function(){
		try {
			var pl = new SOAPClientParameters();
			pl.add("token", createjiraissue.token);
			SOAPClient.invoke(createjiraissue.soapurl, "getProjectsNoSchemes", pl, false, createjiraissue.parseProjectResult);
		} catch (e) {
			createjiraissue.promptService.alert(window, "Error", "Error while retrieving projects.");
		}
    },

    parseProjectResult: function(result, xml){
        //createjiraissue.printXml(xml);

        // get the IDs of all of the project objects
        var projXmlIds = new Array();
        var hrefs = xml.getElementsByTagName("getProjectsNoSchemesReturn");
        for (var i = 0; i < hrefs.length; i++) {
            var attrs = hrefs[i].attributes;
            for (var j = 0; j < attrs.length; j++) {
                if (attrs[j].name == "href") {
                    projXmlIds[i] = (new String(attrs[j].value)).substr(1);
                    //createjiraissue.LOG("parseProjectResult projXmlIds[i]: " + projXmlIds[i]);
                }
            }
        }

        createjiraissue.projects = new Array();

        var projects = xml.getElementsByTagName("multiRef");
        var projKeySize = 0;

        for (var k = 0; k < projects.length; k++) {
            var project = projects[k];
            var projAttrs = project.attributes;
            var multirefId = "";
            for (var m = 0; m < projAttrs.length; m++) {
                if (projAttrs[m].name == "id") {
                    multirefId = projAttrs[m].value;
                }
            }
            for (var n = 0; n < projXmlIds.length; n++) {
                if (multirefId == projXmlIds[n]) {
                    // got a real project
                    var projId = project.getElementsByTagName("id").item(0).firstChild.nodeValue;
                    var projKey = project.getElementsByTagName("key").item(0).firstChild.nodeValue;
                    var projName = project.getElementsByTagName("name").item(0).firstChild.nodeValue;
                    //createjiraissue.LOG("Got project with key: " + projKey + ", id: " + projId + " and name: " + projName);
                    createjiraissue.projects[projKeySize++] = new Array(projName, projKey, projId);
                }
            }
        }
    },

    jiraCreateIssue: function(issueSubject, issueDescription, projId, itId, assignToMe){
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
		prefs = prefs.getBranch("extensions.createjiraissue.");
		try {
			var rIssue = new Array();
			rIssue["project"] = projId;
			rIssue["type"] = itId;
			rIssue["summary"] = issueSubject;
			rIssue["description"] = issueDescription;
			rIssue["reporter"] = createjiraissue.username;
			if (assignToMe) {
				rIssue["assignee"] = createjiraissue.username;
			}
			var pl = new SOAPClientParameters();
			pl.add("token", createjiraissue.token);
			pl.add("issue", rIssue);
			SOAPClient.invoke(createjiraissue.soapurl, "createIssue", pl, false, createjiraissue.parseIssueCreateResult);
		}
		catch (e) {
			createjiraissue.promptService.alert(window, "Error", "Could not create issue. Probably a required field for the seclected issue type is missing.");
		}
    },

    parseIssueCreateResult: function(result, xml){
		//createjiraissue.printXml(xml);
		var issueKey = xml.getElementsByTagName("key").item(0).firstChild.nodeValue;
		try {
			//createjiraissue.alertService.showAlertNotification("chrome://createjiraissue/skin/jira_med.png", "Info", "The issue has been created successfully:\n\n<a href=" + createjiraissue.jiraurl + "/browse/" + issueKey + ">" + issueKey + "</a>");
			createjiraissue.alertService.showAlertNotification("chrome://createjiraissue/skin/jira_med.png", "Info", "The issue " + issueKey + " has been created successfully.");
		} catch(exc) {
			// no alertService, Mac OS X ?
			createjiraissue.LOG(exc);
			createjiraissue.promptService.alert(window, "Info", "The issue has been created successfully:\n\n"+ createjiraissue.jiraurl +"/browse/"+ issueKey);
		}
    },

    jiraAddComment: function(issueDescription, issueKey){
		try {
			var rComment = new Array();
			rComment["body"] = issueDescription;
			var pl = new SOAPClientParameters();
			pl.add("token", createjiraissue.token);
			pl.add("issueKey", issueKey);
			pl.add("comment", rComment);
			SOAPClient.invoke(createjiraissue.soapurl, "addComment", pl, false);
			try {
				//createjiraissue.alertService.showAlertNotification("chrome://createjiraissue/skin/jira_med.png", "Info", "The comment has been added successfully:\n\n<a href=" + createjiraissue.jiraurl + "/browse/" + issueKey + ">" + issueKey + "</a>");
				createjiraissue.alertService.showAlertNotification("chrome://createjiraissue/skin/jira_med.png", "Info", "The comment has been added successfully to " + issueKey + ".");
			} catch(exc) {
				createjiraissue.LOG(exc);
				createjiraissue.promptService.alert(window, "Info", "The comment has been added successfully:\n\n"  + createjiraissue.jiraurl+"/browse/"+issueKey);
			}

		} catch (e) {
			createjiraissue.promptService.alert(window, "Error", "Error occurred. Could not add the comment.");
		}
    },

	setCheckboxAssignToMe: function(){
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
		prefs = prefs.getBranch("extensions.createjiraissue.");
		document.getElementById("checkboxAssignToMe").checked = prefs.getBoolPref("assigntome");
	},

	setCheckboxAdditionalHeader: function(){
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
		prefs = prefs.getBranch("extensions.createjiraissue.");
		document.getElementById("checkboxAdditionalHeaderInformation").checked = prefs.getBoolPref("addheader");
		createjiraissue.addAdditionalHeader();
	},

	addAdditionalHeader: function(){
		if (document.getElementById("checkboxAdditionalHeaderInformation").checked == true) {
			document.getElementById("issueDescription").value = window.arguments[0].issueDescriptionPrefix + window.arguments[0].issueDescription;
		} else {
			document.getElementById("issueDescription").value = window.arguments[0].issueDescription;
		}
	},

    printXml: function(doc){
        var s = new XMLSerializer();
        createjiraissue.LOG(s.serializeToString(doc));
    },

    LOG: function(msg){
        var consoleService = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);
        consoleService.logStringMessage(msg);
    }
};
